class LoginFailure(Exception):
    def __init__(self, *args, **kwargs):
        Exception.__init__(self, *args, **kwargs)


class noTokenForUsername(Exception):
    def __init__(self, username):
        self.username = username
        Exception.__init__(self, f"Can't login as user ({self.username})")


class CreatePlaylistFailure(Exception):
    def __init__(self, results):
        Exception.__init__(self, results)


class PlaylistDeleteFailed(Exception):
    def __init__(self, results):
        Exception.__init__(self, results)


class FailedToCopyPlaylist(Exception):
    def __init__(self, results):
        Exception.__init__(self, results)


class PlaylistNotFound(Exception):
    def __init__(self, playlist_id):
        Exception.__init__(self, playlist_id)


class FailedToAddToPlaylist(Exception):
    def __init__(self, results):
        Exception.__init__(self, results)



from datetime import datetime
import spotipy
import spotipy.util as util
import sharp_darwin.exceptions
from collections import defaultdict


class SharpDarwin:
    def __init__(self, username=None, scope=None, credCache=None):
        self.scope = "user-follow-read user-read-playback-state user-top-read playlist-read-private playlist-modify-private playlist-modify-public playlist-read-collaborative"
        if scope:
            self.scope = scope
        self.username = username
        self.token = None
        self.client = None
        self.args = None
        self.credCache = credCache

    def timestamp(self):
        dt = datetime.now()
        return dt.isoformat()

    def artistsFollowed(self, limit=20):
        res = self.client.current_user_followed_artists(limit)

        artists = []

        # defaultdict(int) provides a value of 0 for all new members
        genres = defaultdict(int)

        for artist in res["artists"]["items"]:
            artists.append({
                "name": artist["name"],
                "totalFollowers": artist["followers"]["total"],
                "id": artist["id"],
                "genres": artist["genres"],
                "popularity": artist["popularity"]
            })

            for genre in artist["genres"]:
                genres[genre] += 1

        return {
                "timestamp": self.timestamp(),
                "artists": artists, 
                "genres": genres
        }

    def audioAnalysis(self, trackID):
        res = self.client.audio_analysis(trackID)
        return res

    def deviceList(self):
        res = self.client.devices()
        return res

    def currentPlayback(self):
        res = self.client.current_playback()

        if res is None:
            return None

        if res["context"]:
            context = {"type": res["context"]["type"],
                       "id": res["context"]["uri"].split(":")[-1]}
            context["name"] = self.getPlaylistName(context["id"])

        else:
            context = None

        device = res["device"]

        artists = []
        for artist in res["item"]["album"]["artists"]:
            artists.append({"name": artist["name"], "id": artist["id"]})

        album = {
            "type": res["item"]["album"]["album_type"],
            "id": res["item"]["album"]["id"],
            "name": res["item"]["album"]["name"],
            "release_date": res["item"]["album"]["release_date"],
            "total_tracks": res["item"]["album"]["total_tracks"]
        }

        track = {
            "name": res["item"]["name"],
            "id": res["item"]["id"],
            "popularity": res["item"]["popularity"],
        }

        state = {
            "progress_ms": res["progress_ms"],
            "repeat_state": res["repeat_state"],
            "shuffle_state": res["shuffle_state"],
            "timestamp": res["timestamp"]
        }

        output = {
            "timestamp": self.timestamp(),
            "context": context,
            "device": device,
            "artists": artists,
            "album": album,
            "track": track,
            "state": state
        }

        return output

    def getPlaylistName(self, playlist_id):
        # Get playlist name
        playlistName = None
        try:
            res = self.client.user_playlist(
                user=self.username, playlist_id=playlist_id, fields="name")
            return res["name"]
        except BaseException:
            raise PlaylistNotFound(playlist_id)

    def login(self):
        if not self.token:
            self.token = util.prompt_for_user_token(
                username=self.username, scope=self.scope, 
                cache_path=self.credCache)
        try: 
            if self.token:
                self.client = spotipy.Spotify(auth=self.token)
            else:
                raise noTokenForUsername(self.username)
        except spotipy.client.SpotifyException as e:
            raise exceptions.LoginFailure(e)
        return True

    def me(self):
        return self.client.me()

    def playlistCopy(self, source, target):
        # Counter for total tracks copied
        count = 0

        # Add tracks. This bit is in a sub-funtion to support the
        # 100 track limit of user_playlist_add_tracks
        def addTracks(target, tracks):
            res = self.client.user_playlist_add_tracks(
                user=self.username, playlist_id=target, tracks=tracks)
            # Retun of a snapshot ID == Success
            if "snapshot_id" in res:
                return True
            else:
                raise FailedToCopyPlaylist(res)

        # Get a track list of the source playlist
        tracks = []
        # Limit to 100... user_playlist_add_tracks can only add 100 tracks at a
        # time
        res = self.client.user_playlist_tracks(
            user=self.username, playlist_id=source, limit=100)
        for track in res["items"]:
            # Add tracks to list
            tracks.append(track["track"]["id"])

        try:
            # Actually add the track list to the target playlist
            addTracks(target, tracks)
            # incr the counter
            count = count + len(tracks)
            # reset the tracks list
            tracks = []
        except BaseException:
            raise

        # Pagination, if needed
        while res["next"]:
            res = self.client.next(res)
            for track in res["items"]:
                # Add tracks to list
                tracks.append(track["track"]["id"])

            try:
                # Actually add the track list to the target playlist
                addTracks(target, tracks)
                # incr the counter
                count = count + len(tracks)
                # reset the tracks list
                tracks = []
            except BaseException:
                raise

        # Return results
        return {
            "timestamp": self.timestamp(),
            "source": source, 
            "target": target, 
            "count": count
        }

    def playlistCreate(self, playlistName, public, descr=None):
        # Create the playlist
        res = self.client.user_playlist_create(
            user=self.username,
            name=playlistName,
            public=public)
            #description=descr)

        # Check for success
        if "id" in res:
            # Return an object of data
            return {
                "timestamp": self.timestamp(),
                "Success": True,
                "playlist-id": res["id"],
                "playlist-name": res["name"],
                "description": res["description"],
                "public": res["public"],
                "collaborative": res["collaborative"]}
        else:
            # Raise failure
            raise CreatePlaylistFailure(res)

    def playlistDelete(self, playlist_id):
        res = self.client.user_playlist_unfollow(
            user=self.username, playlist_id=playlist_id)
        if not res:
            return True
        else:
            raise PlaylistDeleteFailed(res)


    def playlistList(self, mine=False):
        # Array of paginated playlist returns
        lists = []
        # Consolidation of pagination
        items = []
        # Final output
        output = []

        # Make initial requet for user's playlists
        playlists = self.client.user_playlists(self.username)
        lists.append(playlists)

        # Pagination
        while playlists["next"]:
            playlists = self.client.next(playlists)
            lists.append(playlists)

        # Consolidate pagination
        for lst in lists:
            for item in lst["items"]:
                items.append(item)

        # Loop though playlists
        for playlist in items:
            # Skip playlists not created by the user
            if mine:
                if playlist["owner"]["id"] != self.username:
                    continue
            # Get the total numer of tracks
            total = int(playlist["tracks"]["total"])
            # Create an object for each playlist
            output.append({
                "owner": playlist["owner"]["id"],
                "id": playlist["id"],
                "total": total,
                "playlistName": playlist["name"]
            })
        # Done!
        return {
            "timestamp": self.timestamp(),
            "count": len(output), 
            "data": output
        }

    def topArtists(self, limit, time_range):
        try:
            res = self.client.current_user_top_artists(
                limit=limit, time_range=time_range)
        except BaseException:
            raise

        artists = []

        # defaultdict(int) provides a value of 0 for all new members
        genresDict = defaultdict(int)

        for artist in res["items"]:
            artists.append({
                "name": artist["name"],
                "genres": artist["genres"], 
                "popularity": artist["popularity"],
                "url": artist["external_urls"]["spotify"]
            })

            for genre in artist["genres"]:
                genresDict[genre] += 1


        genres = [ {"genre": x, "count": genresDict[x]} for x in genresDict ]

        output = {"artists": artists, "genres": genres}
        return {
            "timestamp": self.timestamp(),
            "count": len(artists),
            "time_range": time_range,
            "data": output}

    def topTracks(self, limit, time_range):
        try:
            res = self.client.current_user_top_tracks(
                limit=limit, time_range=time_range)
        except BaseException:
            raise

        tracks = []

        for track in res["items"]:
            artists = []
            for artist in track["artists"]:
                artists.append(artist["name"])
            tracks.append({
                "uri":  track["uri"],
                "url": track["external_urls"]["spotify"],
                "trackName": track["name"],
                "popularity": track["popularity"],
                "artists": artists,
                "trackID": track["id"],
                "releaseDate": track["album"]["release_date"],
                "albumName": track["album"]["name"]
            }
            )

        output = {"tracks": tracks}
        return {
            "timestamp": self.timestamp(),
            "count": len(tracks),
            "time_range": time_range,
            "data": output}

    def trackList(self, playlist_id):
        # Get playlist name
        playlistName = self.getPlaylistName(playlist_id)

        tracks = []

        # Get initial track list
        res = self.client.user_playlist_tracks(
            user=self.username, playlist_id=playlist_id)
        for track in res["items"]:
            tracks.append(track["track"]["id"])

        # Paginate
        while res["next"]:
            res = self.client.next(res)
            for track in res["items"]:
                tracks.append(track["track"]["id"])

        # object containing final count and data
        output = {
            "timestamp": self.timestamp(),
            "count": len(tracks),
            "playlistName": playlistName,
            "playlist_id": playlist_id,
            "tracks": []}

        # Parse the track data
        trackData = []
        for item in res["items"]:
            track = item["track"]
            artists = []

            href = track["href"]
            albumName = track["name"]
            trackName = track["name"]
            trackID = track["id"]
            popularity = track["popularity"]
            addedAt = item["added_at"]

            for a in track["artists"]:
                artists.append(a["name"])

            trackData.append({
                "artists": artists,
                "albumName": albumName,
                "trackName": trackName,
                "popularity": popularity,
                "trackID": trackID,
                "addedAt": addedAt,
                "href": href
            })
        # Add track data to the final results
        output["tracks"] = trackData
        return output

    def tracksAdd(self, playlist_id, trackID):
        # Add track to playlist
        if trackID is True:
            trackInfo = self.currentPlayback()
            trackID = trackInfo["track"]["id"]

        self.addTrackToPlaylist(playlist_id, [trackID])
        return True

    def addTrackToPlaylist(self, playlist_id, tracks):
        # Add one (str) or more (list) tracks to a playlist
        res = self.client.user_playlist_add_tracks(
            user=self.username, playlist_id=playlist_id, tracks=tracks)
        # Retun of a snapshot ID == Success
        if "snapshot_id" in res:
            return True
        else:
            raise FailedToAddToPlaylist

    def newReleases(self, country, limit=50, next=None):
        data = []
        res = self.client.new_releases(country=country, limit=limit)
        data.append(res)
        
        while res["albums"]["next"]:
            res = self.next(res["albums"])
            data.append(res)
        return data
        
    def next(self, url):
        return self.client.next(url)




#!/usr/bin/env python3

from sys import exit
from sharp_darwin.frontend import *
from sharp_darwin.utils import argParser

from sharp_darwin.frontend import *
from sharp_darwin.utils import argParser

def main():
    ###########################
    # Setup                   #
    ###########################
    # Load up the cli args
    parser = argParser()
    args = parser.parse_args()
    sharpDarwin = init(args)
    sharpDarwin.args = args

    ###########################
    # Commands                #
    ###########################
    if args.command == "artists-followed":
        # Listing of artists followed
        artistsFollowed(sharpDarwin)

    elif args.command == "audio-analysis":
        # Audio analysis for a specified track
        audioAnalysis(sharpDarwin)

    elif args.command == "current-playback":
        # Current playback
        currentPlayback(sharpDarwin)

    elif args.command == "device-list":
        deviceList(sharpDarwin)

    elif args.command == "me":
        # Return Spotify account info
        me(sharpDarwin)

    elif args.command == "playlist-copy":
        # Copy a playlist to another
        playlistCopy(sharpDarwin)

    elif args.command == "playlist-create":
        # Create a playlist
        playlistCreate(sharpDarwin)

    elif args.command == "playlist-delete":
        # Delete a playlist
        playlistDelete(sharpDarwin)

    elif args.command == "playlist-list":
        # Get a curated listing of playlists
        playlistList(sharpDarwin)

    elif args.command == "top-artists":
        # Get user's top artists
        topArtists(sharpDarwin)

    elif args.command == "top-tracks":
        # Get user's top tracks
        topTracks(sharpDarwin)

    elif args.command == "tracks-list":
        # List tracks in a playlist
        tracksList(sharpDarwin)

    elif args.command == "tracks-add":
        # Add track to playlist
        tracksAdd(sharpDarwin)

    elif args.command == "new-releases":
        # List new releases
        newReleases(sharpDarwin)

    else:
        # This should never happen... unless I made a mistake.
        parser.print_help()
        print("\nUnexpected command? Given: ", args.command)


if __name__ == "__main__":
    main()



import os
from pathlib import Path
from sharp_darwin.SharpDarwin import SharpDarwin
from dotenv import load_dotenv
from sharp_darwin.utils import jsonPrint


def init(args):
    # Ensure python can support f-strings
    try:
        xyz = eval('f""')
    except SyntaxError:
        print(
            "This app uses Python f-strings. Please upgrade to or run using Python >= 3.6")
        exit(1)
    except BaseException:
        raise

    # Dotenv env loader
    env_path = Path('.') / '.env'
    load_dotenv(dotenv_path=env_path)

    # Get the username from env
    if "SPOTIPY_USERNAME" not in os.environ:
        print("Missing envionment variable 'SPOTIPY_USERNAME'")
        print(
            "Add your username to the .env file or export SPOTIPY_USERNAME='your-username'")
        exit(1)

    username = os.environ["SPOTIPY_USERNAME"]

    credCache = None
    if args.cache:
        credCache = args.cache

    # Used for calls out to spotify
    sharpDarwin = SharpDarwin(username=username, credCache=credCache)

    # Log onto Spotify
    try:
        sharpDarwin.login()
    except BaseException:
        raise

    return sharpDarwin


def artistsFollowed(sharpDarwin):
    res = sharpDarwin.artistsFollowed(sharpDarwin.args.limit)
    jsonPrint(res)


def audioAnalysis(sharpDarwin):
    res = sharpDarwin.audioAnalysis(sharpDarwin.args.id[0])
    jsonPrint(res)


def currentPlayback(sharpDarwin):
    res = sharpDarwin.currentPlayback()
    # jsonPrint(res)

    if res is None:
        if sharpDarwin.args.json:
            jsonPrint({"success": False,
                       "message": "nothing is currently playing"})
        else:
            print("Nothing currently playing")
    else:
        if sharpDarwin.args.json:
            jsonPrint(res)
        else:
            track = [
                {"Name": res["track"]["name"]},
                {"Popularity": res["track"]["popularity"]},
                {"ID": res["track"]["id"]}
            ]

            album = [
                {"Name": res["album"]["name"]},
                {"Release Date": res["album"]["release_date"]},
                {"ID": res["album"]["id"]}
            ]

            artists = []
            for artist in res["artists"]:
                artists.append(artist["name"])

            playlist = None
            if res["context"]:
                playlistName = res["context"]["name"]
                playlistID = res["context"]["id"]
                playlist = [
                    {"Playlist Name": playlistName},
                    {"Playlist ID": playlistID}
                ]

            device = [
                {"Device Name": res["device"]["name"]},
                {"Device Type": res["device"]["type"]},
                {"Volume": res["device"]["volume_percent"]},
                {"Device ID": res["device"]["id"]}
            ]
            from datetime import datetime
            state = [
                {"Progress (ms)": res["state"]["progress_ms"]},
                {"Repeat": res["state"]["repeat_state"]},
                {"Suffle": res["state"]["shuffle_state"]},
                {"Timestamp": datetime.fromtimestamp(res["state"]["timestamp"] / 1000).isoformat()}
            ]

            labelWidth = 13

            print()
            print("Track info")
            print("==========")
            print(f"    {'Artists':{labelWidth}s} : {', '.join(artists)}")
            for i in track:
                for x, y in i.items():
                    print(f"    {x:{labelWidth}s} : {y}")

            print()
            print("Album info")
            print("==========")
            for i in album:
                for x, y in i.items():
                    print(f"    {x:{labelWidth}s} : {y}")

            print()
            print("Playlist")
            print("========")
            if not playlist:
                print("    (no playlist)")
            else:
                for i in playlist:
                    for x, y in i.items():
                        print(f"    {x:{labelWidth}s} : {y}")

            print()
            print("Device Info")
            print("==========")
            for i in device:
                for x, y in i.items():
                    print(f"    {x:{labelWidth}s} : {y}")

            print()
            print("State Info")
            print("==========")
            for i in state:
                for x, y in i.items():
                    print(f"    {x:{labelWidth}s} : {y}")


def deviceList(sharpDarwin):
    res = sharpDarwin.deviceList()
    jsonPrint(res)


def me(sharpDarwin):
    """ Show basic user data """
    ret = sharpDarwin.me()
    if sharpDarwin.args.json:
        jsonPrint(ret)
    else:
        displayName = ret["display_name"]
        followers = ret["followers"]["total"]
        href = ret["href"]
        userID = ret["id"]
        userType = ret["type"]
        uri = ret["uri"]
        print(f"{'Display Name':12s}: {displayName}")
        print(f"{'Followers':12s}: {followers}")
        print(f"{'href':12s}: {href}")
        print(f"{'User ID':12s}: {userID}")
        print(f"{'User Type':12s}: {userType}")
        print(f"{'URI':12s}: {uri}")


def playlistCopy(sharpDarwin):
    """ Copy a playlist to another """
    source = sharpDarwin.args.source[0]
    target = sharpDarwin.args.target[0]

    try:
        jsonPrint(sharpDarwin.playlistCopy(source=source, target=target))
    except BaseException:
        raise


def playlistCreate(sharpDarwin):
    """ Create a playlist """
    playlistName = sharpDarwin.args.name[0]
    public = False
    descr = ""

    if sharpDarwin.args.public:
        public = True

    if sharpDarwin.args.descr:
        descr = sharpDarwin.args.descr[0]

    # Create the playlist
    try:
        jsonPrint(
            sharpDarwin.playlistCreate(
                playlistName=playlistName,
                public=public,
                descr=descr
            )
        )
    except BaseException:
        raise


def playlistDelete(sharpDarwin):
    """ Delete a playlist """
    try:
        sharpDarwin.playlistDelete(sharpDarwin.args.id[0])
        if sharpDarwin.args.json:
            jsonPrint({"success": True})
        else:
            print("Playlist deleted")
    except BaseException:
        raise


def playlistList(sharpDarwin):
    """ Display playlists """
    data = sharpDarwin.playlistList(mine=sharpDarwin.args.mine)

    if sharpDarwin.args.json:
        jsonPrint(data)
    else:
        # Get the longest char len
        # to format the output correctly
        longest = 0
        for playlist in data["data"]:
            charLength = len(playlist["owner"])
            if charLength > longest:
                longest = charLength

        # Output header
        print(f"{'id':^22s} | {'owner':^{longest}s} | {'total':^4s} | name")
        # Display data
        for playlist in data["data"]:
            total = int(playlist["total"])
            print(
                f"{playlist['id']} | {playlist['owner']:{longest}s} | {total:>5d} | {playlist['playlistName']}")


def topArtists(sharpDarwin):
    """ Show top artist details for the user """
    if sharpDarwin.args.time == "short":
        time_range = "short_term"
    if sharpDarwin.args.time == "med":
        time_range = "medium_term"
    if sharpDarwin.args.time == "long":
        time_range = "long_term"

    res = sharpDarwin.topArtists(
        limit=sharpDarwin.args.limit,
        time_range=time_range)
    if sharpDarwin.args.json:
        jsonPrint(res)
    else:
        count = res["count"]
        time_range = res["time_range"]
        artists = res["data"]["artists"]
        genres = sorted(
            res["data"]["genres"],
            reverse=True,
            key=lambda x: x["count"])

        longestArtist = 0
        for artist in artists:
            artistName = len(artist["name"])
            if artistName > longestArtist:
                longestArtist = artistName

        itemCount = 0
        print("\n===Artists===\n")
        print(f"rank | {'artists':^{longestArtist}s} | genres")
        for artist in artists:
            itemCount = itemCount + 1
            artistName = artist["name"]
            artistGenres = artist["genres"]
            print(
                f"#{itemCount:3d} | {artistName:{longestArtist}s} | {', '.join(artistGenres)}")

        print("\n===Genres===\n")
        longestGenre = 0
        for genre in genres:
            if len(genre["genre"]) > longestGenre:
                longestGenre = len(genre["genre"])
        print(f"{'genre':^{longestGenre}s} | occurrences")
        for genre in genres:
            print(f"{genre['genre']:{longestGenre}s} | {genre['count']}")


def topTracks(sharpDarwin):
    """ Show top artist details for the user """
    if sharpDarwin.args.time == "short":
        time_range = "short_term"
    if sharpDarwin.args.time == "med":
        time_range = "medium_term"
    if sharpDarwin.args.time == "long":
        time_range = "long_term"

    res = sharpDarwin.topTracks(
        limit=sharpDarwin.args.limit,
        time_range=time_range)

    if sharpDarwin.args.json:
        jsonPrint(res)
    else:
        longestTrack = 0
        longestAlbum = 0
        longestArtist = 0
        for track in res["data"]["tracks"]:
            trackName = len(track["trackName"])
            albumName = len(track["albumName"])
            artists = len(", ".join(track["artists"]))

            if trackName > longestTrack:
                longestTrack = trackName
            if albumName > longestAlbum:
                longestAlbum = albumName
            if artists > longestArtist:
                longestArtist = artists

        print(f"{'track':^{longestTrack}s} | {'album':^{longestAlbum}s} | {'artists':^{longestArtist}s} | {'release date':^12s} | {'popularity':^10s} | {'track ID':^22s}")
        for track in res["data"]["tracks"]:
            releaseDate = "------------"
            if track["releaseDate"]:
                releaseDate = track["releaseDate"]
            print(f"{track['trackName']:{longestTrack}s} | {track['albumName']:{longestAlbum}s} | {', '.join(track['artists']):{longestArtist}s} | {releaseDate:12s} | {track['popularity']:<10d} | {track['trackID']}")


def tracksList(sharpDarwin):
    """ List tracks in a playlist """
    playlist_id = sharpDarwin.args.id[0]
    res = sharpDarwin.trackList(playlist_id)

    if sharpDarwin.args.json:
        jsonPrint(res)
    else:

        # Get max field widths
        longestArtist = 0
        longestAlbum = 0
        longestAddedAtDate = 0
        longestTrackName = 0
        for track in res["tracks"]:
            artists = len(", ".join(track["artists"]))
            album = len(track["albumName"])
            trackName = len(track["trackName"])

            if artists > longestArtist:
                longestArtist = artists
            if album > longestAlbum:
                longestAlbum = album
            if trackName > longestTrackName:
                longestTrackName = trackName

        print(f"{'artists':^{longestArtist}s} | {'album':^{longestAlbum}s} | {'track name':^{longestTrackName}s} | {'date added':^20s} | {'popularity':^10s} | {'track ID':^22s}")
        for track in res["tracks"]:
            addedAt = "------------"
            if track["addedAt"]:
                addedAt = track["addedAt"]

            print(f"{', '.join(track['artists']):{longestArtist}s} | {track['albumName']:{longestAlbum}s} | {track['trackName']:{longestTrackName}s} | {addedAt:20s} | {track['popularity']:<10d} | {track['trackID']}")


def tracksAdd(sharpDarwin):
    if sharpDarwin.args.now:
        tid = True
    else:
        tid = sharpDarwin.args.id[0]
    jsonPrint(sharpDarwin.tracksAdd(sharpDarwin.args.playlist_id[0], tid))


def newReleases(sharpDarwin):
    data = sharpDarwin.newReleases(country=sharpDarwin.args.country)
    if sharpDarwin.args.json:
        jsonPrint(data)
    else:
        for s in data:
            for item in s["albums"]["items"]:
                artists = [artist["name"] for artist in item["artists"]]                
                print(item["name"])
                print(f"  Artists:      {', '.join(artists)}")
                print(f"  Type:         {item['album_type']}")
                print(f"  Release Date: {item['release_date']}")
                print(f"  Total Tracks: {item['total_tracks']}")
                print(f"  Spotify ID:   {item['id']}")
                print()
            
        
    



import json
from os.path import abspath
import argparse

# Various random things which are useful


def jsonPrint(data):
    """ Pretty print json data """
    print(
        json.dumps(
            data,
            sort_keys=True,
            indent=4,
            separators=(',', ': ')
        )
    )


def writeJSONFile(filename, data, pretty=False):
    """ Write json data to file """
    fqpn = abspath(filename)
    with open(fqpn, "w") as f:
        if pretty:
            json.dump(
                data,
                f,
                sort_keys=True,
                indent=4,
                separators=(
                    ',',
                    ': '))
        else:
            json.dump(data, f)


def argParser():
    parser = argparse.ArgumentParser(description="Spotify Playlist Manager")
    subparsers = parser.add_subparsers(
        help='Command help',
        dest="command")
        #required=True)

    ###############
    # Global args #
    ###############
    parser.add_argument(
        "--json",
        action="store_true",
        help="Display output as json")

    parser.add_argument(
        "--cache",
        type=str,
        help="Location of credentials cache")

    ################
    # Sub-commands #
    ################

    """ artists followed """
    sp_cmd_artists_followed = subparsers.add_parser(
        "artists-followed", help="Listing of artists followed")
    sp_cmd_artists_followed.add_argument(
        "--limit", type=int, default=20, help="List limit (default: 20)")

    """" audio analysis """
    sp_cmd_audio_analysis = subparsers.add_parser(
        "audio-analysis", help="Audio analysis for a track")
    sp_cmd_audio_analysis.add_argument(
        "--id", type=str, nargs=1, required=True, help="Track ID")

    """ current playback """
    sp_cmd_current_playback = subparsers.add_parser(
        "current-playback", help="Show the current playback")

    """ device List """
    sp_cmd_device_list = subparsers.add_parser(
        "device-list", help="List devices")

    """ me """
    sp_cmd_me = subparsers.add_parser("me", help="Show user info")

    """ playlist-copy """
    sp_cmd_playlist_copy = subparsers.add_parser(
        "playlist-copy", help="Copy tracks from source to target")
    sp_cmd_playlist_copy.add_argument(
        "--source", type=str,
        nargs=1, required=True,
        help="Source playlist ID from which to copy tracks")
    sp_cmd_playlist_copy.add_argument(
        "--target", type=str,
        nargs=1, required=True,
        help="Target playlist ID to copy tracks to")

    """ playlist-create """
    sp_cmd_playlist_create = subparsers.add_parser(
        "playlist-create", help="Create playlists")
    sp_cmd_playlist_create.add_argument(
        "--name",
        type=str,
        nargs=1,
        required=True,
        help="Playlist Name")
    sp_cmd_playlist_create.add_argument(
        "--public",
        action="store_true",
        help="Make playlist public")
    sp_cmd_playlist_create.add_argument(
        "--descr", type=str, nargs=1, help="Optional description")

    """ playlist-delete """
    sp_cmd_playlist_delete = subparsers.add_parser(
        "playlist-delete", help="Delete playlists")
    sp_cmd_playlist_delete.add_argument(
        "--id",
        type=str,
        nargs=1,
        required=True,
        help="ID of playlist")
    sp_cmd_playlist_delete.add_argument(
        "--confirm",
        action="store_true",
        required=True,
        help="Confirm this list will be deleted")

    """ playlist-list """
    sp_cmd_playlist_show = subparsers.add_parser(
        "playlist-list", help="Lists playlists")
    sp_cmd_playlist_show.add_argument(
        "--mine",
        action="store_true",
        default=False,
        help="Shows only my playlists")

    """ top-artists """
    sp_cmd_top_artists = subparsers.add_parser(
        "top-artists", help="Show user's top artists")
    sp_cmd_top_artists.add_argument(
        "--limit", type=int,
        default=50,
        help="Number of results to return (default/max: 50)")
    sp_cmd_top_artists.add_argument(
        "--time", choices=["short", "med", "long"],
        default="long",
        help="Choose timeframe [short, med, long] (default: long)")

    """ top-tracks """
    sp_cmd_top_tracks = subparsers.add_parser(
        "top-tracks", help="Show user's top tracks")
    sp_cmd_top_tracks.add_argument(
        "--limit", type=int,
        default=50,
        help="Number of results to return (default/max: 50)")
    sp_cmd_top_tracks.add_argument(
        "--time", choices=["short", "med", "long"],
        default="long",
        help="Choose timeframe [short, med, long] (default: long)")

    """ tracks-list """
    sp_cmd_tracks_list = subparsers.add_parser(
        "tracks-list", help="List tracks in a playlist")
    sp_cmd_tracks_list.add_argument(
        "--id",
        type=str,
        nargs=1,
        required=True,
        help="Playlist ID")

    """ tracks-add """
    sp_cmd_tracks_list = subparsers.add_parser(
        "tracks-add", help="Adds a track to a playlist")
    sp_cmd_tracks_list.add_argument(
        "--playlist-id",
        required=True,
        type=str, nargs=1,
        help="Playlist ID")
    mutex = sp_cmd_tracks_list.add_mutually_exclusive_group(required=True)
    mutex.add_argument(
        "--id",
        type=str,
        nargs=1,
        help="Track ID")
    mutex.add_argument(
        "--now",
        action="store_true",
        help="Add the current song")

    """ new-releases """
    sp_cmd_new_releases = subparsers.add_parser(
        "new-releases", help="Lists new releases"
    )
    sp_cmd_new_releases.add_argument(
        "-c", "--country", default="US", type=str,
        help="An ISO 3166-1 alpha-2 country code"
    )

    return parser



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="sharp_darwin",
    version="1.3.4",
    author="Robert Sigler",
    author_email="sigler@improvisedscience.org",
    description="Spotify Playlist Manager",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/rmrfslashbin/sharp-darwin",
    packages=["sharp_darwin"],
    install_requires=[
        "spotipy>=2.10",
        "python-dotenv>=0.12"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.6",
    entry_points = {
        "console_scripts": [
            "sharp-darwin=sharp_darwin.sharp_darwin_cli:main"
        ]
    }
)
